package xyz.brassgoggledcoders.moarcarts.entities;

import xyz.brassgoggledcoders.moarcarts.MoarCarts;
import xyz.brassgoggledcoders.moarcarts.fakeworld.FakePlayer;
import xyz.brassgoggledcoders.moarcarts.fakeworld.FakeWorld;
import xyz.brassgoggledcoders.moarcarts.network.EntityTileEntityUpdateMessage;
import xyz.brassgoggledcoders.moarcarts.renderers.IRenderBlock;
import net.minecraft.block.BlockContainer;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.ITickable;
import net.minecraft.world.World;
import xyz.brassgoggledcoders.boilerplate.lib.BoilerplateLib;

/**
 * @author SkySom
 */
public abstract class EntityMinecartTEBase extends EntityMinecartBase implements IRenderBlock
{
	protected TileEntity tileEntity;

	private static int IS_DIRTY_DW = 30;
	private static int IS_CLIENT_NEEDY = 28;
	private static int UPDATE_TICKS = 200;

	public EntityMinecartTEBase(World world, int metadata)
	{
		super(world, metadata);
		if(this.getCartBlock() instanceof BlockContainer)
		{
			this.setTileEntity(this.getCartBlock().createTileEntity(world, this.func_174897_t()));
		}
		this.setDirty(true);
	}

	@Override
	public void func_70088_a()
	{
		super.func_70088_a();
		field_70180_af.func_75682_a(IS_DIRTY_DW, (byte) 1);
		field_70180_af.func_75682_a(IS_CLIENT_NEEDY, (byte) 1);
	}

	@Override
	public void func_70071_h_()
	{
		super.func_70071_h_();
		if(!this.field_70170_p.field_72995_K && (this.isClientNeedy() ||
				(this.isDirty() && random.nextInt(UPDATE_TICKS) == 0)))
		{
			this.setDirty(false);
			this.setClientNeedy(false);
			this.sendUpdateToAllAround();
		}
		if(shouldTileUpdate() && this.getTileEntity() instanceof ITickable)
		{
			((ITickable)this.getTileEntity()).func_73660_a();
		}
	}

	@Override
	public boolean func_130002_c(EntityPlayer player)
	{
		this.sendUpdateToAllAround();
		if(!player.func_70093_af())
		{
			EntityPlayer fakePlayer = new FakePlayer(player, this, this.shouldAccessPlayerInventory());
			return this.getCartBlock().func_180639_a(this.getFakeWorld(), ORIGIN_POS, this.func_174897_t(),
					fakePlayer, EnumFacing.NORTH, 0, 0, 0);
		}
		return true;
	}

	public boolean shouldAccessPlayerInventory()
	{
		return false;
	}


	@Override
	protected void func_70037_a(NBTTagCompound nbtTagCompound)
	{
		super.func_70037_a(nbtTagCompound);
		this.getTileEntity().func_145839_a(nbtTagCompound.func_74775_l("TILEENTITY"));
		this.setDirty(nbtTagCompound.func_74767_n("DIRTY"));
		this.setClientNeedy(nbtTagCompound.func_74767_n("NEEDY"));
	}

	@Override
	protected void func_70014_b(NBTTagCompound nbtTagCompound)
	{
		super.func_70014_b(nbtTagCompound);
		NBTTagCompound tileEntityNBTTagCompound = new NBTTagCompound();
		this.getTileEntity().func_145841_b(tileEntityNBTTagCompound);
		nbtTagCompound.func_74782_a("TILEENTITY", tileEntityNBTTagCompound);
		nbtTagCompound.func_74757_a("DIRTY", this.isDirty());
		nbtTagCompound.func_74757_a("NEEDY", this.isClientNeedy());
	}

	public void markDirty()
	{
		this.setDirty(true);
	}

	public void setTileEntityNBT(ItemStack itemStack)
	{
		if(itemStack.func_77942_o() && this.shouldSaveDataToItem())
		{
			this.getTileEntity().func_145839_a(itemStack.func_77978_p().func_74775_l("tilenbt"));
		}
	}

	@Override
	public RenderMethod getRenderMethod()
	{
		return this.getItem().getCartBlockRenderMethod(this.getCartItem());
	}

	public void sendUpdateToAllAround()
	{
		if(!field_70170_p.field_72995_K)
		{
			BoilerplateLib.getInstance().packetHandler.sendToAllAround(new EntityTileEntityUpdateMessage(this), this);
		}
	}

	@Override
	public void afterEntitySpawned()
	{
		this.sendUpdateToAllAround();
	}

	public boolean shouldSaveDataToItem()
	{
		return false;
	}

	public void dropCart(ItemStack cartItem)
	{
		if(this.shouldSaveDataToItem())
		{
			NBTTagCompound nbtTagCompound = new NBTTagCompound();
			NBTTagCompound itemNBTTagCompound;
			this.getTileEntity().func_145841_b(nbtTagCompound);
			if(cartItem.func_77942_o())
			{
				itemNBTTagCompound = cartItem.func_77978_p();
			} else
			{
				itemNBTTagCompound = new NBTTagCompound();
			}
			this.getTileEntity().func_145841_b(nbtTagCompound);
			itemNBTTagCompound.func_74782_a("tilenbt", nbtTagCompound);
			cartItem.func_77982_d(itemNBTTagCompound);
		}
		super.dropCart(cartItem);
	}

	public TileEntity createTileEntity()
	{
		if(field_70170_p != null && this.getCartBlock() != null)
		{
			return this.getCartBlock().createTileEntity(field_70170_p, this.func_174897_t());
		}
		return null;
	}

	public TileEntity getTileEntity()
	{
		if(this.tileEntity == null)
		{
			this.setTileEntity(this.createTileEntity());
		}
		return tileEntity;
	}

	public void setTileEntity(TileEntity tileEntity)
	{
		NBTTagCompound nbtTagCompound = null;
		if(this.tileEntity != null)
		{
			nbtTagCompound = new NBTTagCompound();
			this.tileEntity.func_145841_b(nbtTagCompound);
		}

		this.tileEntity = tileEntity;
		if(this.tileEntity != null)
		{
			this.setFakeWorld(new FakeWorld(this));
			this.tileEntity.func_145834_a(this.getFakeWorld());
			if(nbtTagCompound != null)
			{
				this.tileEntity.func_145839_a(nbtTagCompound);
			}
			this.sendUpdateToAllAround();
		} else
		{
			MoarCarts.logger.error("Null Tile Entity Reported. THIS IS BAD!");
		}
	}

	@Override
	public void func_174899_a(IBlockState blockState)
	{
		super.func_174899_a(blockState);

		TileEntity tileEntity = this.createTileEntity();
		if(tileEntity != null)
		{
			this.setTileEntity(tileEntity);
		}
	}

	public boolean shouldTileUpdate()
	{
		return false;
	}

	public boolean isDirty()
	{
		return field_70180_af.func_75683_a(IS_DIRTY_DW) != 0;
	}

	public void setDirty(boolean isDirty)
	{
		field_70180_af.func_75692_b(IS_DIRTY_DW, isDirty ? 1 : (byte) 0);
	}

	public boolean isClientNeedy()
	{
		return field_70180_af.func_75683_a(IS_CLIENT_NEEDY) != 0;
	}

	public void setClientNeedy(boolean needy)
	{
		field_70180_af.func_75692_b(IS_CLIENT_NEEDY, needy ? 1 : (byte) 0);
	}

	public boolean showHalo()
	{
		return false;
	}

	public String getHaloString()
	{
		return "";
	}
}
